/// <reference path="../../reference/cristalix.d.ts" />
var __extends = (this && this.__extends) || (function () {
    var extendStatics = function (d, b) {
        extendStatics = Object.setPrototypeOf ||
            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
            function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
        return extendStatics(d, b);
    };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
import { colorParts2Hex } from './colors';
import { runningAnimations, Animation } from './fast-gui';
import * as index from './index';
export var buffer = GLAllocation.createDirectFloatBuffer(16);
var glPushMatrix = GL11.glPushMatrix;
var glPopMatrix = GL11.glPopMatrix;
var glMultMatrix = GL11.glMultMatrix;
var glColor = GL11.glColor4f;
var AbstractElement = /** @class */ (function () {
    function AbstractElement(data) {
        var offset = data.offset, scale = data.scale, align = data.align, origin = data.origin, rotation = data.rotation, enabled = data.enabled, color = data.color;
        this.enabled = enabled == null ? true : enabled;
        var matrices = new Array(index.matrixFields);
        var properties = new Array(index.valueFields);
        // NativeRuntime.fillIntArray(properties, 0, properties.length, 0);
        for (var i = 0; i < properties.length; i++) {
            properties[i] = 0;
        }
        this.properties = properties;
        this.matrices = matrices;
        if (offset)
            this.setOffset(offset);
        this.setProperty(index.scaleX, scale && scale.x ? scale.x : 1);
        this.setProperty(index.scaleY, scale && scale.y ? scale.y : 1);
        this.setProperty(index.scaleZ, scale && scale.z ? scale.z : 1);
        if (align)
            this.setAlign(align);
        if (origin)
            this.setOrigin(origin);
        if (rotation && rotation.x)
            this.setProperty(index.rotationX, rotation.x);
        if (rotation && rotation.y)
            this.setProperty(index.rotationY, rotation.y);
        this.setProperty(index.rotationZ, rotation ? (rotation.z || 0) : 1);
        if (rotation && rotation.angle)
            this.setProperty(index.rotationAngle, rotation.angle);
        if (color) {
            this.setProperty(index.colorA, color.a == undefined ? 1 : color.a);
            this.setProperty(index.colorR, color.r == undefined ? 1 : color.r);
            this.setProperty(index.colorG, color.g == undefined ? 1 : color.g);
            this.setProperty(index.colorB, color.b == undefined ? 1 : color.b);
        }
        this.setProperty(index.parentSizeX, 0);
        this.setProperty(index.parentSizeY, 0);
        this.setProperty(index.parentSizeZ, 0);
        // );
        this.onClick = data.onClick || null;
        this.onHover = data.onHover || null;
        this.beforeRender = data.beforeRender || null;
        this.afterRender = data.afterRender || null;
    }
    ;
    AbstractElement.prototype.applyTransformations = function () {
        if (!this.enabled)
            return;
        this.cleanMatrices();
        for (var i = 0; i < index.matrixFields; i++) {
            var matrix = this.matrices[i];
            if (matrix) {
                matrix.store(buffer);
                buffer.flip();
                glMultMatrix(buffer);
            }
        }
    };
    AbstractElement.prototype.setOffset = function (offset, duration, easingId, onFinish) {
        var x = offset.x;
        var y = offset.y;
        var z = offset.z;
        if ((x !== undefined) && (x === x))
            this.setProperty(index.offsetX, x, duration, easingId, onFinish);
        if ((y !== undefined) && (y === y))
            this.setProperty(index.offsetY, y, duration, easingId, onFinish);
        if ((z !== undefined) && (z === z))
            this.setProperty(index.offsetZ, z, duration, easingId, onFinish);
    };
    AbstractElement.prototype.addOffset = function (offset, duration, easingId, onFinish) {
        this.setOffset({
            x: this.properties[index.offsetX] + offset.x,
            y: this.properties[index.offsetY] + offset.y,
            z: this.properties[index.offsetZ] + offset.z,
        }, duration, easingId, onFinish);
    };
    AbstractElement.prototype.setAlign = function (align, duration, easingId, onFinish) {
        var x = align.x;
        var y = align.y;
        var z = align.z;
        if (x !== undefined && x === x)
            this.setProperty(index.alignX, x, duration, easingId, onFinish);
        if (y !== undefined && y === y)
            this.setProperty(index.alignY, y, duration, easingId, onFinish);
        if (z !== undefined && z === z)
            this.setProperty(index.alignZ, z, duration, easingId, onFinish);
    };
    AbstractElement.prototype.addAlign = function (align, duration, easingId, onFinish) {
        this.setAlign({
            x: this.properties[index.alignX] + align.x,
            y: this.properties[index.alignY] + align.y,
            z: this.properties[index.alignZ] + align.z,
        }, duration, easingId, onFinish);
    };
    AbstractElement.prototype.setOrigin = function (origin, duration, easingId, onFinish) {
        var x = origin.x;
        var y = origin.y;
        var z = origin.z;
        if (x !== undefined && x === x)
            this.setProperty(index.originX, x, duration, easingId, onFinish);
        if (y !== undefined && y === y)
            this.setProperty(index.originY, y, duration, easingId, onFinish);
        if (z !== undefined && z === z)
            this.setProperty(index.originZ, z, duration, easingId, onFinish);
    };
    AbstractElement.prototype.addOrigin = function (origin, duration, easingId, onFinish) {
        this.setOrigin({
            x: this.properties[index.originX] + origin.x,
            y: this.properties[index.originY] + origin.y,
            z: this.properties[index.originZ] + origin.z,
        }, duration, easingId, onFinish);
    };
    AbstractElement.prototype.setScale = function (scale, duration, easingId, onFinish) {
        var x = scale.x;
        var y = scale.y;
        var z = scale.z;
        if (x !== undefined && x === x)
            this.setProperty(index.scaleX, x, duration, easingId, onFinish);
        if (y !== undefined && y === y)
            this.setProperty(index.scaleY, y, duration, easingId, onFinish);
        if (z !== undefined && z === z)
            this.setProperty(index.scaleZ, z, duration, easingId, onFinish);
    };
    AbstractElement.prototype.addScale = function (scale, duration, easingId, onFinish) {
        this.setAlign({
            x: this.properties[index.scaleX] + scale.x,
            y: this.properties[index.scaleY] + scale.y,
            z: this.properties[index.scaleZ] + scale.z,
        }, duration, easingId, onFinish);
    };
    AbstractElement.prototype.setColor = function (color, duration, easingId, onFinish) {
        var a = color.a;
        var r = color.r;
        var g = color.g;
        var b = color.b;
        if (a !== undefined && a === a)
            this.setProperty(index.colorA, a, duration, easingId, onFinish);
        if (r !== undefined && r === r)
            this.setProperty(index.colorR, r, duration, easingId, onFinish);
        if (g !== undefined && g === g)
            this.setProperty(index.colorG, g, duration, easingId, onFinish);
        if (b !== undefined && b === b)
            this.setProperty(index.colorB, b, duration, easingId, onFinish);
    };
    AbstractElement.prototype.addColor = function (color, duration, easingId, onFinish) {
        this.setColor({
            a: this.properties[index.colorA] + color.a,
            r: this.properties[index.colorR] + color.r,
            g: this.properties[index.colorG] + color.g,
            b: this.properties[index.colorB] + color.b,
        }, duration, easingId, onFinish);
    };
    AbstractElement.prototype.setRotation = function (rotation, duration, easingId, onFinish) {
        var x = rotation.x;
        var y = rotation.y;
        var z = rotation.z;
        var angle = rotation.angle;
        if (angle !== undefined && angle == angle)
            this.setProperty(index.rotationAngle, angle, duration, easingId);
        if (x !== undefined && x === x)
            this.setProperty(index.rotationX, x, duration, easingId);
        if (y !== undefined && y === y)
            this.setProperty(index.rotationY, y, duration, easingId);
        if (z !== undefined && z === z)
            this.setProperty(index.rotationZ, z, duration, easingId);
    };
    AbstractElement.prototype.addRotation = function (angle, duration, easingId, onFinish) {
        this.setProperty(index.rotationAngle, angle, duration, easingId, onFinish);
    };
    AbstractElement.prototype.setProperty = function (propertyId, value, duration, easingId, onFinish) {
        if (duration) {
            var animation = void 0;
            for (var _i = 0, runningAnimations_1 = runningAnimations; _i < runningAnimations_1.length; _i++) {
                var existing = runningAnimations_1[_i];
                if (existing.element === this && existing.propertyId === propertyId) {
                    animation = existing;
                    break;
                }
            }
            if (!animation)
                animation = new Animation(this, propertyId);
            animation.newTarget(value, duration, easingId, onFinish);
            runningAnimations.push(animation);
            return;
        }
        // stdout.println('setting ' + propertyId + ' to ' + value);
        this.properties[propertyId] = value;
        var influence = index.matrixInfluence[propertyId];
        if (influence.length) {
            this.markDirty(influence);
        }
    };
    AbstractElement.prototype.markDirty = function (matrixIds) {
        if (!this.dirtyMatrices)
            this.dirtyMatrices = [];
        loop: for (var _i = 0, matrixIds_1 = matrixIds; _i < matrixIds_1.length; _i++) {
            var matrixId = matrixIds_1[_i];
            for (var _a = 0, _b = this.dirtyMatrices; _a < _b.length; _a++) {
                var dirtyMatrix = _b[_a];
                if (dirtyMatrix == matrixId)
                    continue loop;
            }
            // stdout.println('Marking ' + matrixId + ' as dirty');
            this.dirtyMatrices.push(matrixId);
        }
    };
    AbstractElement.prototype.cleanMatrices = function () {
        var dirty = this.dirtyMatrices;
        if (!dirty || !dirty.length)
            return;
        for (var _i = 0, dirty_1 = dirty; _i < dirty_1.length; _i++) {
            var dirtyMatrixId = dirty_1[_i];
            if (dirtyMatrixId == -128)
                continue;
            this.updateMatrix(dirtyMatrixId);
        }
        this.dirtyMatrices = null;
    };
    AbstractElement.prototype.updateMatrix = function (matrixId) {
        // stdout.println('updating matrix ' + matrixId)
        var properties = this.properties;
        if (this.dirtyMatrices) {
            this.dirtyMatrices[this.dirtyMatrices.indexOf(matrixId)] = -128;
        }
        if (matrixId == index.colorMatrix) {
            this.cachedHexColor = colorParts2Hex(properties[index.colorA], properties[index.colorR], properties[index.colorG], properties[index.colorB]);
        }
        if (matrixId >= 0) {
            var matrix = new Matrix4f().setIdentity();
            switch (matrixId) {
                case index.alignMatrix:
                    matrix.translate(new Vector3f(properties[index.alignX] * properties[index.parentSizeX], properties[index.alignY] * properties[index.parentSizeY], properties[index.alignZ] * properties[index.parentSizeZ]));
                    break;
                case index.rotationMatrix:
                    matrix.rotate(properties[index.rotationAngle], new Vector3f(properties[index.rotationX], properties[index.rotationY], properties[index.rotationZ]));
                    break;
                case index.offsetMatrix:
                    matrix.translate(new Vector3f(properties[index.offsetX], properties[index.offsetY], properties[index.offsetZ]));
                    break;
                case index.scaleMatrix:
                    matrix.scale(new Vector3f(properties[index.scaleX], properties[index.scaleY], properties[index.scaleZ]));
                    break;
                case index.originMatrix:
                    matrix.translate(new Vector3f(-properties[index.originX] * properties[index.sizeX], -properties[index.originY] * properties[index.sizeY], -properties[index.originZ] * properties[index.sizeZ]));
                    break;
            }
            this.matrices[matrixId] = matrix;
        }
    };
    AbstractElement.prototype.transformAndRender = function () {
        if (!this.enabled)
            return;
        glPushMatrix();
        this.cleanMatrices();
        this.applyTransformations();
        if (this.beforeRender)
            this.beforeRender();
        this.render();
        if (this.afterRender)
            this.afterRender();
        glPopMatrix();
    };
    return AbstractElement;
}());
export { AbstractElement };
var TextElement = /** @class */ (function (_super) {
    __extends(TextElement, _super);
    function TextElement(data) {
        var _this = _super.call(this, data) || this;
        _this.text = data.text || "";
        _this.shadow = !!data.shadow;
        _this.autoFit = !!data.autoFit;
        _this.setProperty(index.sizeX, fontRenderer.getStringWidth(_this.text));
        _this.properties[index.sizeY] = 9;
        return _this;
    }
    TextElement.prototype.updateMatrix = function (matrixId) {
        if ((matrixId == index.sizeMatrix || matrixId == index.scaleMatrix) && this.autoFit) {
            var textWidth = this.properties[index.sizeX];
            var parentWidth = this.properties[index.parentSizeX];
            if (textWidth) {
                var factor = (parentWidth - 2) / textWidth;
                if (factor < 1)
                    this.properties[index.scaleX] = factor;
            }
        }
        _super.prototype.updateMatrix.call(this, matrixId);
    };
    TextElement.prototype.setText = function (text) {
        this.text = text;
        this.setProperty(index.sizeX, Draw.getStringWidth(text));
    };
    TextElement.prototype.render = function () {
        // GL11.glDepthFunc(GL11.GL_LESS);
        // GlStateManager.disableDepth();
        // ToDo: colored text
        fontRenderer.drawString(this.text, 0, fontRenderer.getUnicodeFlag() ? 0 : 1, -1, this.shadow);
        // GL11.glDepthFunc(GL11.GL_LEQUAL);
        // GlStateManager.enableDepth();
    };
    return TextElement;
}(AbstractElement));
export { TextElement };
var RectangleElement = /** @class */ (function (_super) {
    __extends(RectangleElement, _super);
    function RectangleElement(data) {
        var _this = _super.call(this, data) || this;
        if (data.size) {
            if (data.size.x)
                _this.setProperty(index.sizeX, data.size.x);
            if (data.size.y)
                _this.setProperty(index.sizeY, data.size.y);
        }
        var texture = data.texture;
        if (texture) {
            var start = texture.start;
            if (start) {
                if (start.x)
                    _this.setProperty(index.textureX, start.x);
                if (start.y)
                    _this.setProperty(index.textureY, start.y);
            }
            _this.texture = texture.resource || null;
        }
        _this.setProperty(index.textureWidth, (texture && texture.size) ? texture.size.x || 1 : 1);
        _this.setProperty(index.textureHeight, (texture && texture.size) ? texture.size.y || 1 : 1);
        _this.children = data.children || [];
        return _this;
    }
    RectangleElement.prototype.updateMatrix = function (matrixId) {
        if (matrixId === index.sizeMatrix) {
            var properties = this.properties;
            for (var _i = 0, _a = this.children; _i < _a.length; _i++) {
                var child = _a[_i];
                var childProperties = child.properties;
                childProperties[index.parentSizeX] = properties[index.sizeX];
                childProperties[index.parentSizeY] = properties[index.sizeY];
                childProperties[index.parentSizeZ] = properties[index.sizeZ];
                child.updateMatrix(index.alignMatrix);
            }
        }
        _super.prototype.updateMatrix.call(this, matrixId);
    };
    RectangleElement.prototype.render = function () {
        var properties = this.properties;
        if (this.texture) {
            GlStateManager.enableBlend();
            Textures.bindTexture(this.texture);
            GL11.glColor4f(1, 1, 1, 1);
            var precision = 1073741824;
            Draw.drawScaledCustomSizeModalRect(0, 0, properties[index.textureX] * precision, properties[index.textureY] * precision, properties[index.textureWidth] * precision, properties[index.textureHeight] * precision, properties[index.sizeX], properties[index.sizeY], precision, precision);
        }
        else {
            Draw.drawRect(0, 0, properties[index.sizeX], properties[index.sizeY], this.cachedHexColor);
        }
        for (var _i = 0, _a = this.children; _i < _a.length; _i++) {
            var child = _a[_i];
            child.transformAndRender();
        }
    };
    RectangleElement.prototype.setSize = function (size, duration, easingId, onFinish) {
        var x = size.x;
        var y = size.y;
        var z = size.z;
        if (x !== undefined && x === x)
            this.setProperty(index.sizeX, x, duration, easingId, onFinish);
        if (y !== undefined && y === y)
            this.setProperty(index.sizeY, y, duration, easingId, onFinish);
        if (z !== undefined && z === z)
            this.setProperty(index.sizeZ, z, duration, easingId, onFinish);
    };
    RectangleElement.prototype.addSize = function (size, duration, easingId, onFinish) {
        this.setSize({
            x: this.properties[index.sizeX] + size.x,
            y: this.properties[index.sizeY] + size.y,
            z: this.properties[index.sizeZ] + size.z,
        }, duration, easingId, onFinish);
    };
    return RectangleElement;
}(AbstractElement));
export { RectangleElement };
var ItemElement = /** @class */ (function (_super) {
    __extends(ItemElement, _super);
    function ItemElement(data) {
        var _this = _super.call(this, data) || this;
        _this.item = data.item;
        _this.setProperty(index.sizeX, 16);
        _this.setProperty(index.sizeY, 16);
        return _this;
    }
    ItemElement.prototype.render = function () {
        RenderHelper.enableGUIStandardItemLighting();
        Draw.renderItemAndEffectIntoGUI(this.item, 0, 0);
        RenderHelper.disableStandardItemLighting();
    };
    return ItemElement;
}(AbstractElement));
export { ItemElement };
